(function() {
	var Grit, EventEmitter;

	/*
	 * require in NodeJS's native EventEmitter class,
	 * or emulate it in the browser.
	 */

	if(typeof exports !== 'undefined') {
		EventEmitter = require('events').EventEmitter;
	} else {
		EventEmitter = function() {
			var handlers = {};

			this.on = this.addListener = function(e, callback) {
				if(!handlers.hasOwnProperty(e)) {
					handlers[e] = [];
				}
				handlers[e].push(callback);
			};

			this.removeListener = function(e, callback) {
				var handleList, index, length, curr;
				if(handlers.hasOwnProperty(e)) {
					handleList = handlers[e];
					for(index = 0, length = handleList.length; index < length; index++) {
						curr = handleList[index];
						if(curr === callback) {
							handleList.splice(index, 1);
							if(handleList.length === 0)
								delete handlers[e];
							return;
						}	
					}
				}
			};

			this.emit = function(e) {
				var handleList, index, length, args;
				if(handlers.hasOwnProperty(e)) {
					handleList = handlers[e];
					args = Array.prototype.slice(this, arguments).slice(1);
					for(index = 0, length = handleList.length; index < length; index++)
						handleList[index].call(this, args);
				}
			};
		};
	}

	/*
	 * The model class itself.
	 */

	Grit = function() {
		EventEmitter.call(this);
		var data = {};
		this.get = function(name) {
			if(data.hasOwnProperty(name))
				return data[name];
			return null;
		};
		this.set = function(name, value, silent) {
			if(typeof silent === 'undefined' || !silent)
				this.emit('change:' + name, this, value);
			data[name] = value;
			this.sync(this, data, name);
			return value;
		};
	};
	Grit.prototype = new EventEmitter();
	Grit.prototype.constructor = Grit;

	// sync is left empty -- redefine it to automate
	// changes being synced to the server.
	Grit.prototype.sync = function(model, data, property) {
	};

	/* 
	 * Export
	 */

	if(typeof exports !== 'undefined') {
		exports = module.exports = Grit;
	} else {
		window.Grit = Grit;
	}
}());
